#ifndef TASK__H
#define TASK__H

#include "pq.h"

#define EVENT_NOERROR		0
#define	EVENT_NOTASKS		-1
#define EVENT_OVERFLOW		-2
#define EVENT_TIMEOUT		-3
#define EVENT_DELETED		-4

/*------------------------------------------------------------------
** Definition of a Task Control Block (TCB)
**----------------------------------------------------------------*/

typedef struct tcb {
	long *stack;			/*	points to current position in stack	*/
	long *stacktop;		/*	points to the end of the stack		*/
	int stacksize;		/*	SIZE of the stack in words			*/
	int priority;		/*	task priority	*/
	int status;			/*	task status, set by semaphore mostly	*/
	long TimeStamp;		/*	system time stamp, set when swapped	*/
	long TcbSwaps;		/*	number of times tasks swapped in	*/
	int timeout;        /*	timeout counter to trigger event	*/
	char *name;			/*	name of task	*/
	struct tcb *next;
	struct tcb *list;	/*	master linked list of tasks	*/
} TCB;

/*-------------------------------------------------------------------
** Semaphore Object
**-----------------------------------------------------------------*/
typedef struct ecb {
	int SemaphoreMode;		//mode of semaphore
	struct ecb *Tnext,*Tprev;	//Link list pointers for timeout list
	int EventCount;			//counter for semaphore
	TCB *task_h;		//head pointer for list of waiting tasks
	TCB *task_t;		//tail pointer for list of waiting tasks
	char name[20];
} ECB;

#define SEMAPHORE_MODE_BLOCKING		0
#define SEMAPHORE_MODE_TIMEOUT		1

/*-------------------------------------------------------------------
** Function Prototypes
**------------------------------------------------------------------*/

extern ECB *MasterList;	//head pointer for master semaphore list
extern ECB *TimeoutList;	//head pointer for timeout semaphore list
extern PQ ActiveTasks;		//priority task queue
extern TCB *pxCurrentTCB,*pxReadyTCB;

extern void CreateTask(TCB *t, void (*task)(void *),long *stack, int stacksize,int priority,char *name,void *arg);
extern void IrqSwap(void);
extern void InitTask(void);
extern void Yeild(void);
extern ECB *NewSemaphore(int InitCount,int Mode,char *name);
extern void CreateSemaphore(ECB *e,int InitCount,int Mode,char *n);
extern int PendSemaphore(ECB *e, int Timeout);	//wait for semaphore availiable
extern int PostSemaphore(ECB *e, int Value);	//signal semaphore availiable
extern void TimerTicker(void);
extern void vPortYieldProcessor(void);
extern void vPortStartFirstTask(void);
extern int TimeDelay(int mSec);

#endif
